/**
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Wei&szlig;enfelser Stra&szlig;e 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.NoSuchElementException;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfiguration;
import de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt;
import de.bsvrz.kex.bwsin.konfig.kb.SINKonfigurationsBereich;
import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: KML.java 10460 2008-07-10 17:28:00Z gieseler $
 * 
 */
public class KML {

	private Okstra odb;
	private ClientDavConnection dav;
	private SINKonfigurationsBereich kb;
	private BufferedOutputStream _bos;
	private int level;

	File kmlfile;
	StringBuffer _sb = new StringBuffer();

	/**
	 * @param kmlfile
	 */
	public KML(File kmlfile) {
		try {
			_bos = new BufferedOutputStream(new FileOutputStream(kmlfile));
		} catch (IOException e) {
			System.err.println("Fehler beim anlegen der Datei: "
					+ kmlfile.getAbsoluteFile());
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	public void appendLine(String line) {
		_sb.append(indent(level));
		_sb.append(line);
		_sb.append("\n");

		// flush
		if (_sb.length() > 10000) {
			writeConfigData(_sb);
			_sb.setLength(0);
		}
	}

	public void declevel() {
		level--;
	}

	public void ende() {
		print_filefooter(_sb);

		try {
			_bos.close();
		} catch (IOException e) {
			System.err.println("Fehler beim Schlieen der Datei.");
			e.printStackTrace();
			throw new RuntimeException(e);
		}

	}

	/**
	 * @param kmlfile
	 */
	private void erzeugeKML(List<KonfigurationsObjekt> kolist, File kmlfile) {
		StringBuffer sb = new StringBuffer();

		try {
			_bos = new BufferedOutputStream(new FileOutputStream(kmlfile));
		} catch (IOException e) {
			System.err.println("Fehler beim anlegen der Datei: "
					+ kmlfile.getAbsoluteFile());
			e.printStackTrace();
			throw new RuntimeException(e);
		}

		Collections.sort(kolist, new Comparator<KonfigurationsObjekt>() {
			public int compare(KonfigurationsObjekt o1, KonfigurationsObjekt o2) {
				return o1.getPid().compareTo(o2.getPid());
			}
		});

		// for( SystemObject so : elemente) {
		// sb = new StringBuffer();
		// WGS84Polygon polygon = new WGS84Polygon(getPunktliste(ak,
		// (ConfigurationObject) so));
		// printData(so, polygon, sb);
		// }
		//		

	}

	private ArrayList<WGS84Punkt> getPunktliste(SINAktuelleKonfiguration ak,
			ConfigurationObject co) {
		ArrayList<WGS84Punkt> punkte = new ArrayList<WGS84Punkt>();

		AttributeGroup ratg = ak.getAttributeGroup("atg.linienKoordinaten"); //$NON-NLS-1$
		Data rd = co.getConfigurationData(ratg);
		if (rd != null) {
			Data.NumberArray x = rd.getScaledArray("x"); //$NON-NLS-1$
			Data.NumberArray y = rd.getScaledArray("y"); //$NON-NLS-1$
			int anzahl = x.getLength();

			for (int i = 0; i < anzahl; i++) {
				WGS84Punkt p = new WGS84Punkt(x.getValue(i).doubleValue(), y
						.getValue(i).doubleValue());
				punkte.add(p);
			}
		}

		return punkte;
	}

	public void inclevel() {
		level++;
	}

	/**
	 * <?xml version="1.0" encoding="UTF-8"?> <kml
	 * xmlns="http://earth.google.com/kml/2.1"> <Document> <name>Paths</name>
	 * <description>Examples of paths. Note that the tessellate tag is by
	 * default set to 0. If you want to create tessellated lines, they must be
	 * authored (or edited) directly in KML.</description> <Style
	 * id="yellowLineGreenPoly"> <LineStyle> <color>7f00ffff</color> <width>4</width>
	 * </LineStyle> <PolyStyle> <color>7f00ff00</color> </PolyStyle> </Style>
	 * <Placemark> <name>Absolute Extruded</name> <description>Transparent
	 * green wall with yellow outlines</description>
	 * <styleUrl>#yellowLineGreenPoly</styleUrl> <LineString> <extrude>1</extrude>
	 * <tessellate>1</tessellate> <altitudeMode>absolute</altitudeMode>
	 * <coordinates> -112.2550785337791,36.07954952145647,2357
	 * -112.2549277039738,36.08117083492122,2357
	 * -112.2552505069063,36.08260761307279,2357
	 * -112.2564540158376,36.08395660588506,2357
	 * -112.2580238976449,36.08511401044813,2357
	 * -112.2595218489022,36.08584355239394,2357
	 * -112.2608216347552,36.08612634548589,2357
	 * -112.262073428656,36.08626019085147,2357
	 * -112.2633204928495,36.08621519860091,2357
	 * -112.2644963846444,36.08627897945274,2357
	 * -112.2656969554589,36.08649599090644,2357 </coordinates> </LineString>
	 * </Placemark> </Document> </kml>
	 */

	private StringBuffer indent(int level) {
		StringBuffer sb = new StringBuffer(level);
		for (int i = 0; i < level; ++i) {
			sb.append("\t");
		}
		return sb;
	}

	public void initDav() {
		String[] commandArguments = {
				"-datenverteiler=127.0.0.1:8083",
				"-benutzer=Tester",
				"-authentifizierung=/home/gysi/workspace/DAV-Neu/Projekte/SIN-Test/skripte-bash/passwd",
				"-debugSetLoggerAndLevel=INFO",
				"-debugSetLoggerAndLevel=de.bsvrz.kex.bwsin:INFO",
				"-debugLevelStdErrText=INFO" };
		ArgumentList argumentList = new ArgumentList(commandArguments);
		try {
			String myname = "SIN-TEST";
			Debug.init(myname, argumentList);
			ClientDavParameters parameters = new ClientDavParameters(
					commandArguments);
			parameters.setApplicationName(myname);
			dav = new ClientDavConnection(parameters);
			dav.connect();
			dav.login();
		} catch (NoSuchElementException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (MissingParameterException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (CommunicationError e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (ConnectionException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (InconsistentLoginException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}

	}

	private void print_filefooter(StringBuffer sb) {
		sb.append("\t</Document>\n");
		sb.append("</kml>\n");
		writeConfigData(sb);
	}

	private void print_fileheader(StringBuffer sb, String name,
			String description) {
		sb.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
		sb.append("<kml xmlns=\"http://earth.google.com/kml/2.1\">\n");
		sb.append("\t<Document>\n");
		sb.append("\t\t<name>" + name + "</name>\n");
		sb.append("\t\t<description>" + description + "</description>\n");
		level = 3;
	}

	public void start(String name, String description) {
		print_fileheader(_sb, name, description);
	}

	/**
	 * Schreibt einen Stringbuffer in Datei.
	 * 
	 * @param sb
	 *            Zu schreibender StringBuffer.
	 */
	private void writeConfigData(StringBuffer sb) {
		try {
			_bos.write(sb.toString().getBytes());
		} catch (IOException e) {
			System.out.println("Fehler beim Schreiben auf Datei: "
					+ kmlfile.getAbsolutePath());
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

}
