/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.ArrayList;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;

/**
 * Konfigurationsobjekt.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: KonfigurationsObjekt.java 8267 2008-04-15 18:21:45Z gieseler $
 */
public interface KonfigurationsObjekt {

	/**
	 * Gibt den zugeh&ouml;rigen Typ im Datenkatalog zur&uuml;ck.
	 * 
	 * @return Datenkatalog-Typ
	 */
	SystemObjectType getDavtype();

	/**
	 * Gibt die Koordinaten zur&uuml;ck.
	 * 
	 * @return Koordinatenliste
	 */
	ArrayList<WGS84Punkt> getKoordinaten();

	/**
	 * Gibt die L&auml;nge zur&uuml;ck.
	 * 
	 * @return L&auml;nge
	 */
	double getLaenge();

	/**
	 * Gibt den Namen zur&uuml;ck.
	 * 
	 * @return Name
	 */
	String getName();

	/**
	 * Gibt die PID zur&uuml;ck.
	 * 
	 * @return PID
	 */
	String getPid();

	/**
	 * Gibt das ProxyConfigurationObject zur&uuml;ck.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * 
	 * @return ProxyConfigurationObject
	 * @throws ImportException
	 *             bei Ausnahmen
	 */
	ProxyConfigurationObject getProxyObjekt(ClientDavConnection dav)
			throws ImportException;

	/**
	 * Setzt die Koordinaten.
	 * 
	 * @param punkte
	 *            Koordinatenpunkte WGS84
	 * 
	 */
	void setKoordinaten(ArrayList<WGS84Punkt> punkte);

	/**
	 * Setzt die L&auml;nge.
	 * 
	 * @param laenge
	 *            L&auml;nge
	 */
	void setLaenge(double laenge);

	/**
	 * Setzt den Namen.
	 * 
	 * @param name
	 *            Name
	 */
	void setName(final String name);

	/**
	 * Setzt die PID.
	 * 
	 * @param pid
	 *            PID
	 */
	void setPid(String pid);

}
