/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.okstra.parser;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Element-Handler WFS-Parser.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINElementHandler.java 10428 2008-07-10 12:09:46Z gieseler $
 * 
 */
public class SINElementHandler {

	/**
	 * Textwert.
	 */
	private StringBuffer text;

	/**
	 * Namespace-URI.
	 */
	private String namespaceURI;

	/**
	 * Name.
	 */
	private String localName;

	/**
	 * Attribute.
	 */
	AttributesImpl attribute = new AttributesImpl();

	/**
	 * Konstruktor.
	 */
	public SINElementHandler() {
		/**
		 * 
		 */
	}

	/**
	 * F&uuml;gt Text hinzu.
	 * 
	 * @param ch
	 *            Zeichenfeld
	 * @param start
	 *            Start
	 * @param length
	 *            L&auml;ge
	 */
	public void addText(final char[] ch, final int start, final int length) {
		if (text == null) {
			text = new StringBuffer();
		}

		text.append(ch, start, length);
	}

	/**
	 * endElement-Callback.
	 * 
	 * @param namespaceuri
	 *            Namespace-URI
	 * @param localname
	 *            Name
	 */
	public void endElement(final String namespaceuri, final String localname) {
		/**
		 * 
		 */
	}

	/**
	 * Gibt die Attribute zur&uuml;ck.
	 * 
	 * @return Attribute
	 */
	public AttributesImpl getAttribute() {
		return attribute;
	}

	// /**
	// * @return
	// */
	// public Element getElement() {
	// return elem;
	// }

	// /**
	// * @see org.geotools.xml.XMLElementHandler#getHandler(java.lang.String,
	// * java.lang.String)
	// */
	// public XMLElementHandler getHandler(URI namespaceURI, String localName,
	// Map hints) throws SAXException {
	// throw new SAXException(
	// "Should not have any children - this is a simpleType");
	// }

	/**
	 * Gibt den Namen des Elementes zur&uuml;ck.
	 * 
	 * @return Name
	 */
	public String getName() {
		return localName;
	}

	/**
	 * Gibt die Namespace-URI des Elementes zur&uuml;ck.
	 * 
	 * @return Namespace-URI
	 */
	protected String getNamespaceURI() {
		return namespaceURI;
	}

	/**
	 * Gibt den Text des Elementes zur&uuml;ck.
	 * 
	 * @return Text
	 */
	public String getText() {
		return (text != null) ? text.toString().trim() : "";
	}

	/**
	 * Gibt den Wert des Elementes zur&uuml;ck.
	 * 
	 * @return Wert
	 */
	public Object getValue() {
		if (getText() == null || getText().length() == 0) {
			for (int i = 0; i < attribute.getLength(); i++) {
				if (attribute.getQName(i).equals("xlink:href")) {
					return attribute.getValue(i);
				}
			}
		} else {
			return getText();
		}

		return null;
	}

	/**
	 * Setzt den Text des Elementes.
	 * 
	 * @param text
	 *            Text
	 */
	public void setText(final String text) {
		this.text = (text != null) ? new StringBuffer(text)
				: new StringBuffer();
	}

	/**
	 * startElement-Callback.
	 * 
	 * @param uri
	 *            URI
	 * @param localname
	 *            Name
	 * @param atts
	 *            Attribute
	 */
	public void startElement(final String uri, final String localname,
			final Attributes atts) {
		// this.attr = new AttributesImpl(attr1);
		namespaceURI = uri;
		this.localName = localname;
		// ("", "srsName", "", "anyURI", "" + g.getSRID());
		for (int i = 0, ii = atts.getLength(); i < ii; i++) {
			attribute.addAttribute(null, null, atts.getQName(i), atts
					.getType(i), atts.getValue(i));
		}
	}
}
