/**
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Wei&szlig;enfelser Stra&szlig;e 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.okstra;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Properties;

import org.geotools.data.DefaultQuery;
import org.geotools.data.FeatureReader;
import org.geotools.data.Query;
import org.geotools.data.Transaction;
import org.geotools.feature.Feature;
import org.geotools.feature.IllegalAttributeException;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraFeatures;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraStrecke;

/**
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TestOkstra.java 10462 2008-07-10 17:28:31Z gieseler $
 * 
 */
public class TestOkstra {

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	private Okstra odb;

	public void initOkstra() {
		Properties props = new Properties();
		Map se = new HashMap();

		// props.put("Pfad",
		// "/home/gysi/Documents/BaW/SIN/VRZ_BW_DSK_MIF_1_1/");

		Map<String, Object> m = new HashMap<String, Object>();
		// String
		// URLstr="http://services.interactive-instruments.de/vrzbw/vrz3-wfs.exe";
		// String URLstr = "file://C:\\Dokumente und Einstellungen\\Steffen\\Eigene Dateien\\workspace\\wfsclient\\okstrads";
		String URLstr="file:/home/gysi/workspace/de.bsvrz.kex.bwsin/okstrads_02_2010";
		URL url;
		try {
			url = new URL(URLstr);
		} catch (MalformedURLException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
			return;
		}
		props.put("SERVER_URL", url);
		props.put("FILESTORE", Boolean.TRUE);

		// props.put(WFSDataStoreFactory.TIMEOUT.key,new Integer(3000000));
		// props.put(WFSDataStoreFactory.PROTOCOL.key, Boolean.FALSE); // false
		// == GET, true == POST
		// props.put(WFSDataStoreFactory.BUFFER_SIZE.key, new Integer(5000000));
		// // false == GET, true == POST

		try {
			odb = new Okstra(props);
		} catch (ImportException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}

	// @Test
	public void testOkstraRef() {
		String ref = "urn:x-vrzbw:Strecke.0900b8a4a1";
		assertEquals("okstra:Strecke", Okstra.getRefFeatureType(ref));
		assertEquals("0900b8a4a1", Okstra.getRefFeatureId(ref));
	}

	// @Test
	public void testOkstraStrassenElemente() {
		Properties props = new Properties();
		Map<String, Feature> se = new HashMap<String, Feature>();

		props.put("Pfad", "/home/gysi/Documents/BaW/SIN/VRZ_BW_DSK_MIF_1_1/");

		Okstra o;
		try {
			o = new Okstra(props);
		} catch (ImportException e1) {
			// TODO Automatisch erstellter Catch-Block
			e1.printStackTrace();
			return;
		}

		Query query = new DefaultQuery(OkstraFeatures.STRASSENELEMENT
				.getFeatureOkstraName());
		FeatureReader reader;
		try {
			reader = o.getFeatureReader(query, Transaction.AUTO_COMMIT);
			while (reader.hasNext()) {
				Feature f = reader.next();
				se.put(f.getID(), f);
			}
		} catch (IOException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (NoSuchElementException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (IllegalAttributeException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}

		assertEquals(3424, se.size());
	}

	@Test
	public void testSpecial() {
		initOkstra();
		
		Query query = new DefaultQuery(OkstraFeatures.STRASSENELEMENT
				.getFeatureOkstraName());
		FeatureReader reader;
		try {
			reader = odb.getFeatureReader(query, Transaction.AUTO_COMMIT);
			while (reader.hasNext()) {
				Feature f = reader.next();
//				se.put(f.getID(), f);
				System.out.println("Feature: " + f);
			}
		} catch (IOException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (NoSuchElementException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (IllegalAttributeException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}
		
		// Object[] test =
		// odb.getSpecialAttributeList2(OkstraFeatures.UEBERHOLVERBOT,
		// "UeberholverbotVRZ.090008bo3f",
		// OkstraUeberholverbot.FAHRZEUGART);
		//		
		// System.out.println("test");

		// String strecke = "Strecke.0900b8c12n";
//		String strecke = "Strecke.0900b8c12n";
//		Object[] aoa = odb.getSpecialAttributeList2(OkstraFeatures.STRECKE,
//				strecke, OkstraStrecke.AOA);
//		Object[] anfang = odb.getSpecialAttributeList2(OkstraFeatures.STRECKE,
//				strecke, OkstraStrecke.STATION_ANFANG);
//		Object[] ende = odb.getSpecialAttributeList2(OkstraFeatures.STRECKE,
//				strecke, OkstraStrecke.STATION_ENDE);
//
//		for (int i = 0; i < aoa.length; i++) {
//			System.out.println("AOA: " + aoa[i] + " Anfang: " + anfang[i]
//					+ " Ende: " + ende[i]);
//		}
	}
}
