/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.vew;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.kex.bwsin.konfig.Abweichungsliste;
import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.kex.bwsin.konfig.KonfigurationsDaten;
import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfiguration;
import de.bsvrz.kex.bwsin.konfig.kb.SINKonfigurationsBereich;
import de.bsvrz.kex.bwsin.konfig.lcl.LCL;
import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptklasse f&uuml;r die SWE 2.BW-SIN.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: BWSIN.java 10384 2008-07-09 12:06:48Z gieseler $
 * 
 */
public class BWSIN implements StandardApplication {
	/**
	 * MAIN-Methode.
	 * 
	 * @param args
	 *            Argumente
	 */
	public static void main(String[] args) {
		Thread
				.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {

					public void uncaughtException(Thread t, Throwable e) {
						Logger.getLogger(getClass().getName()).log(Level.INFO,
								"*** Application wird beendet ***", e); //$NON-NLS-1$
					}

				});

		String gui = null;
		final ArgumentList argumentList = new ArgumentList(args);
		if (argumentList.hasArgument(SINProperties.PROP_KEY_WITH_GUI)) {
			gui = argumentList.fetchArgument(SINProperties.PROP_KEY_WITH_GUI)
					.asString();
			SINProperties.getInstance().setProperty(
					SINProperties.PROP_KEY_WITH_GUI, gui);
		}
		if (gui == null || gui.length() == 0 || (!gui.equalsIgnoreCase("ja"))) {
			StandardApplicationRunner.run(new BWSIN(), args);
		} else {
			StandardApplicationRunner.run(new BWSINgui(), args);
		}
	}

	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private Debug debug;

	/**
	 * OKSTRA-Datenbasis.
	 */
	private Okstra okstraDB;

	/**
	 * Datenverteiler-Verbindung.
	 */
	ClientDavConnection dav;

	/**
	 * Standard-Konstruktor.
	 */
	protected BWSIN() {
		super();
	}

	/**
	 * &Uuml;berpr&uuml;ft, ob alle notwendigen Parameter f&uuml;r den Import
	 * gesetzt sind.
	 * 
	 * @throws ImportException
	 *             bei fehlenden notwendigen Paramertern
	 */
	public void checkParameter() throws ImportException {
		// teste notwendige Parameter
		String[] zutesten = { SINProperties.PROP_KEY_LCL,
				SINProperties.PROP_KEY_KBPFAD, SINProperties.PROP_KEY_KV,
				SINProperties.PROP_KEY_KBPID, SINProperties.PROP_KEY_KBINFO };

		for (int i = 0; i < zutesten.length; i++) {
			String testprop = SINProperties.getInstance().getProperty(
					zutesten[i]);
			if (testprop == null || testprop.length() == 0) {
				throw new ImportException(
						"Der Parameter '" + zutesten[i] + "' fehlt!"); //$NON-NLS-1$//$NON-NLS-2$
			}
		}

		// teste optionale Parameter
		String[] zuwarnen = { SINProperties.PROP_KEY_KBNAME };
		for (int i = 0; i < zuwarnen.length; i++) {
			String testprop = SINProperties.getInstance().getProperty(
					zuwarnen[i]);
			if (testprop == null || testprop.length() == 0) {
				debug
						.warning("Der Parameter '" + zuwarnen[i] + "' ist nicht gesetzt!"); //$NON-NLS-1$//$NON-NLS-2$
			}
		}

		// teste Parameter fr Abweichungsliste
		if (SINProperties.getInstance().istAktiv(SINProperties.PROP_KEY_ABW)) {
			String testprop = SINProperties.getInstance().getProperty(
					SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI);
			if (testprop == null || testprop.length() == 0) {
				throw new ImportException(
						"Der Parameter '" + SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI + "' ist nicht gesetzt!"); //$NON-NLS-1$//$NON-NLS-2$
			}
		}
	}

	/**
	 * F&uuml;hrt die Importoperation aus.
	 * 
	 * @throws Exception
	 *             bei Ausnahmen
	 */
	public void importieren() throws Exception {
		try {
			debug.info("Der Import wird gestartet"); //$NON-NLS-1$
			initOkstra();
			LCL.initialisiere(SINProperties.getInstance().getProperty(
					SINProperties.PROP_KEY_LCL));
			SINKonfigurationsBereich kb = new SINKonfigurationsBereich();
			// bilde den Dateinamen den Konfigurationsbereiches aus
			// Pfad+Pid+".xml"
			File kbfile = new File(SINProperties.getInstance().getProperty(
					SINProperties.PROP_KEY_KBPFAD), SINProperties.getInstance()
					.getProperty(SINProperties.PROP_KEY_KBPID)
					+ ".xml"); //$NON-NLS-1$
			kb.initialisiere(dav, kbfile.toString(), SINProperties
					.getInstance().getProperty(SINProperties.PROP_KEY_KV),
					SINProperties.getInstance().getProperty(
							SINProperties.PROP_KEY_KBPID), SINProperties
							.getInstance().getProperty(
									SINProperties.PROP_KEY_KBNAME),
					SINProperties.getInstance().getProperty(
							SINProperties.PROP_KEY_KBINFO));
			SINAktuelleKonfiguration ak = new SINAktuelleKonfiguration();
			ak.initialisiere(dav, SINProperties.getInstance().getProperty(
					SINProperties.PROP_KEY_KBPID));

			KonfigurationsDaten kd = new KonfigurationsDaten(ak, dav, okstraDB);

			kd.importiere();

			kd.exportKB(kb);

			kb.generiereXML();

			Abweichungsliste abl = new Abweichungsliste(ak, kd.getOjekteMap(),
					kbfile);
			abl.erstellen();

			abl.print(System.out);

			if (SINProperties.getInstance()
					.istAktiv(SINProperties.PROP_KEY_ABW)) {
				abl.inDatei(SINProperties.getInstance().getProperty(
						SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI));
			}
		} catch (Exception e) {
			debug.error(e.getMessage());
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	public void initialize(final ClientDavInterface connection)
			throws Exception {
		debug = Debug.getLogger();
		dav = (ClientDavConnection) connection;

		try {
			startImport();
		} catch (Exception e) {
			throw e;
		}
	}

	/**
	 * Initialisiert das OKSTRA-Modul.
	 * 
	 * @throws ImportException
	 *             wenn das Modul nicht initialisiert werden kann
	 */
	public void initOkstra() throws ImportException {
		Properties okstraProperties = new Properties();

		String propfilename = SINProperties.getInstance().getProperty(
				SINProperties.PROP_KEY_QUELLDATENBASIS);

		try {
			InputStream is;
			if (propfilename == null
					|| propfilename.length() == 0
					|| propfilename
							.equals(SINProperties.DEFAULT_OKSTRA_PARAMETERFILE)) {
				is = ClassLoader
						.getSystemResourceAsStream(SINProperties.DEFAULT_OKSTRA_PARAMETERFILE);
			} else {
				is = new FileInputStream(propfilename);
			}

			okstraProperties.load(is);

			URL url = new URL((String) okstraProperties.get("SERVER_URL"));
			if (url.getProtocol().equalsIgnoreCase("file")) {
				okstraProperties.put("FILESTORE", Boolean.TRUE);
			}

			okstraDB = new Okstra(okstraProperties);
		} catch (IOException e) {
			throw new ImportException(
					"Die Properties-Datei der WFS-Verbindungparameter kann nicht gelesen werden (" + //$NON-NLS-1$
							e.getMessage() + ")"); //$NON-NLS-1$
		}
	}

	/**
	 * Wertet die an die Applikation &uuml;bergebenen Kommandozeilenparameter
	 * aus.<br>
	 * Zus&auml;tzlich zu den Standard-Datenverteiler-Argumenten sind folgende
	 * Argumente m&ouml;glich:
	 * <ul>
	 * <li>-guiStatischerImportNetz=&lt;boolean&gt;, Schalter f&uuml;r
	 * grafische Oberfl&auml;che [ja|nein] (Standardeinstellung: nein), legt
	 * fest, ob die SWE mit oder ohne grafischem Interface gestartet wird.</li>
	 * <li>-netzDatenQuelleParameter=&lt;Datei&gt;, Dateiname Properties-Datei
	 * mit den Parametern zum Zugriff auf den WebFeatureService
	 * (Standardeinstellung: wfs_bw_Default.properties)</li>
	 * <li>-dateiNameLcl=&lt;Datei&gt;, Dateiname Location Code Liste</li>
	 * <li>-pfadNameKonfigurationsBereich=&lt;Pfad&gt;, Pfadname f&uuml;r den
	 * zu erzeugenden Konfigurationsbereich, der Dateiname wird aus der Pid
	 * (siehe Aufrufparameter -KonfigurationsBereichsPid) und Endung .xml
	 * erzeugt. Die Datei wird dann an den hier festgelegten Ort geschrieben.</li>
	 * <li>-KonfigurationsVerantwortlicher=&lt;Text&gt;,
	 * Konfigurationsverantwortlicher des zu erzeugenden Konfigurationsbereichs</li>
	 * <li>-KonfigurationsBereichsPid=&lt;Text&gt;, PID des
	 * Konfigurationsbereichs, z. B. 'kb.verkehrStra&szlig;enSegmenteBw'</li>
	 * <li>-KonfigurationsBereichsName=&lt;Text&gt;, Name des
	 * Konfigurationsbereichs, z. B. 'Teilmodell Verkehr Global, Anteil
	 * Stra&szlig;enSegmente, BW'</li>
	 * <li>-KonfigurationsBereichsInfo=&lt;Text&gt;, Infotext des
	 * Konfigurationsbereichs, z. B. Teilmodell Verkehr Global (Anteile
	 * Stra&szlig;enSegmente) f&uuml;r Baden-W&uuml;rttemberg, erzeugt aus der
	 * Stra&szlig;endatenbank.</li>
	 * <li>-mitAbweichungen=&lt;boolean&gt;, Erzeugung von Abweichungslisten
	 * [ja|nein] (Standardeinstellung: nein), legt fest, ob beim Import die neu
	 * erzeugte Konfiguration mit der bestehenden Konfiguration verglichen wird
	 * und in die Protokolldatei die entsprechende Abweichungsliste erzeugt
	 * werden soll.</li>
	 * <li>-dateiNameProtokoll=&lt;Datei&gt;, Dateiname Protokolldatei,
	 * Pfad/Name f&uuml;r die zu erzeugende Protokolldatei mit den Abweichungen
	 * bzw. dem Protokoll des Importvorgangs, z.B.
	 * 'D:/Netz/Abweichungsliste_22.01.2007-15:05.txt'</li>
	 * <li>-netzReferenzen=&lt;Datei&gt;, Dateiname einer java-Datei mit zu
	 * &uuml;berpr&uuml;fenden Referenzen, Standard ist die zur SWE
	 * geh&ouml;rende Datei 'NetzReferenzenDefault'. In der Datei werden alle zu
	 * &uuml;berpr&uuml;fenden Referenzen des Datenkataloges auf Objekte der
	 * Teilmodells Verkehr Global definiert.</li>
	 * <li>-generiereTMCOrtsReferenzen=&lt;boolean&gt;, Erzeugung von
	 * TMCOrtsreferenzen im Konfigurationsbereich [ja|nein]
	 * (Standardeinstellung: ja) , Erlaubt die Unterdr&uuml;ckung der Versorgung
	 * des Attributes 'TmcPrim&auml;rOrt' der Attributgruppe
	 * 'atg.&auml;u&szlig;eresStraenSegment' der Konfigurationsobjekte vom Typ
	 * 'typ.&auml;u&szlig;eresStraenSegment'. Durch die Unterdr&uuml;ckung
	 * dieses Attributes ist es m&ouml;glich, den von dieser SWE erzeugten
	 * Konfigurationsbereich ohne den Konfigurationsbereich, welcher die
	 * TMC-Orte definiert, in den Datenkatalog zu integrieren.</li>
	 * </ul>
	 * 
	 * @param argumentList
	 *            die Liste der &uuml;bergebenen Argumente
	 * @throws Exception
	 *             bei Ausnahmen
	 */
	public void parseArguments(final ArgumentList argumentList)
			throws Exception {
		String[] myargs = { SINProperties.PROP_KEY_WITH_GUI,
				SINProperties.PROP_KEY_QUELLDATENBASIS,
				SINProperties.PROP_KEY_LCL, SINProperties.PROP_KEY_KBPFAD,
				SINProperties.PROP_KEY_KV, SINProperties.PROP_KEY_KBPID,
				SINProperties.PROP_KEY_KBNAME, SINProperties.PROP_KEY_KBINFO,
				SINProperties.PROP_KEY_ABW,
				SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI,
				SINProperties.PROP_KEY_REFERENZEN,
				SINProperties.PROP_KEY_TMCREFERENZEN };

		for (int i = 0; i < myargs.length; i++) {
			if (argumentList.hasArgument(myargs[i])) {
				SINProperties.getInstance().setProperty(myargs[i],
						argumentList.fetchArgument(myargs[i]).asString());
			}
		}

	}

	/**
	 * F&uuml;hrt die Importoperation aus.
	 * 
	 * @throws Exception
	 *             bei Ausnahmen
	 */
	public void startImport() throws Exception {
		try {
			debug = Debug.getLogger();

			checkParameter();

			importieren();
		} catch (Exception e) {
			debug.error(e.getMessage());
		}

		debug.info("Applikation beendet"); //$NON-NLS-1$
		dav.disconnect(false, "");
	}
}
